/*
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 */

/*
 * $Log: netdebug.c,v $
 * Revision 1.5  2002/06/26 17:29:36  harald
 * First pre-release with 2.4 stack
 *
 */

#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/icmp.h>
#include <netinet/ip_icmp.h>
#include <netinet/ipcsum.h>
#include <net/netdebug.h>
#include <sys/socket.h>

#include <sys/print.h>

extern TCPSOCKET *tcpSocketList;
extern UDPSOCKET *udpSocketList;

char tcp_trace = 0;

void NutKDumpTcpHeader(u_char *ds, TCPSOCKET *sock, NETBUF *nb)
{
    TCPHDR *th = (TCPHDR *)nb->nb_tp.vp;

    NutPrintFormat(0, "%s%04X[%u]-SEQ(%lx)", ds, sock, nb->nb_ap.sz, ntohl(th->th_seq));
    if(th->th_flags & TH_ACK) 
        NutPrintFormat(0, "-ACK(%lx)", ntohl(th->th_ack));
    if(th->th_flags & TH_FIN)
        NutPrintString(0, "-FIN");
    if(th->th_flags & TH_SYN)
        NutPrintString(0, "-SYN");
    if(th->th_flags & TH_RST)
        NutPrintString(0, "-RST");
    if(th->th_flags & TH_PUSH)
        NutPrintString(0, "-PSH");
    if(th->th_flags & TH_URG)
        NutPrintString(0, "-URG");
    NutPrintString(0, "\r\n");
}

void NutKDumpSockState(u_char state, u_char *lead, u_char *trail)
{
    if(lead)
        NutPrintString(0, lead);
    switch(state) {
    case TCPS_LISTEN:
        NutPrintString(0, "LISTEN");
        break;
    case TCPS_SYN_SENT:
        NutPrintString(0, "SYNSENT");
        break;
    case TCPS_SYN_RECEIVED:
        NutPrintString(0, "SYNRCVD");
        break;
    case TCPS_ESTABLISHED:
        NutPrintString(0, "ESTABL");
        break;
    case TCPS_FIN_WAIT_1:
        NutPrintString(0, "FINWAIT1");
        break;
    case TCPS_FIN_WAIT_2:
        NutPrintString(0, "FINWAIT2");
        break;
    case TCPS_CLOSE_WAIT:
        NutPrintString(0, "CLOSEWAIT");
        break;
    case TCPS_CLOSING:
        NutPrintString(0, "CLOSING");
        break;
    case TCPS_LAST_ACK:
        NutPrintString(0, "LASTACK");
        break;
    case TCPS_TIME_WAIT:
        NutPrintString(0, "TIMEWAIT");
        break;
    case TCPS_CLOSED:
        NutPrintString(0, "CLOSED");
        break;
    default:
        NutPrintString(0, "?UNK?");
        break;
    }
    if(trail)
        NutPrintString(0, trail);
}


void NutKDumpSocketList(void)
{
    TCPSOCKET *ts;
    UDPSOCKET *us;

    NutPrintString(0, "\r\nSock Typ Local                  Remote                 State\r\n");
    /*                   1234 123 123456789012345:123456 123456789012345:123456 */
    for(ts = tcpSocketList; ts; ts = ts->so_next) {
        NutPrintFormat(0, "%04X TCP %15s:%-6u ", ts, 
                        inet_ntoa(ts->so_local_addr), ntohs(ts->so_local_port));
        NutPrintFormat(0, "%15s:%-6u ", inet_ntoa(ts->so_remote_addr), 
                                      ntohs(ts->so_remote_port));
        NutKDumpSockState(ts->so_state, 0, "\r\n");
    }
    for(us = udpSocketList; us; us = us->so_next) {
        NutPrintFormat(0, "%04X UDP %6u\r\n", us, ntohs(us->so_local_port));
    }
}


/*!
 * \brief Switch on/off TCP trace.
 */
void NutKTcpTrace(char enable)
{
    tcp_trace = enable;
}

