#include <stdio.h>
#include <conio.h>
#include <windows.h>

#include "ispport.h"

/*
 *  1(STROBE): BASE2-0x01 NC
 *  2(DATA0) : BASE0-0x01 KEY1
 *  3(DATA1) : BASE0-0x02 KEY2
 *  4(DATA2) : BASE0-0x04 ENA1 (RESET, MISO)
 *  5(DATA3) : BASE0-0x08 ENA2 (SCK, MOSI, LED)
 *  6(DATA4) : BASE0-0x10 SCK
 *  7(DATA5) : BASE0-0x20 MOSI
 *  8(DATA6) : BASE0-0x40 LED
 *  9(DATA7) : BASE0-0x80 RESET
 * 10(ACK)   : BASE1-0x40 MISO
 * 11(BUSY)  : BASE1-0x80 KEY2
 * 12(PE)    : BASE1-0x20 KEY1
 * 13(SLCT)  : BASE1-0x10 NC
 * 14(AFEED) : BASE2-0x02 NC
 * 15(ERROR) : BASE1-0x08 NC
 * 16(INIT)  : BASE2-0x04 NC
 * 17(SLCTIN): BASE2-0x08 NC
 * 18(GND)   : GND
 * 19(GND)   : GND
 * 20(GND)   : GND
 * 21(GND)   : GND
 * 22(GND)   : GND
 * 23(GND)   : GND
 * 24(GND)   : GND
 * 25(GND)   : GND
 */

static int port_byte;

void IspDelay(void)
{
    volatile int i;

    for(i = 0; i < 1000; i++);
}

/*! 
 * \brief Set SCK line.
 * 
 * \param b If 0 line is set low, otherwise high.
 */
void IspPortClock(int b)
{
    if(b) 
        port_byte |= 0x10; 
    else 
        port_byte &= ~0x10;
    _outp(0x378, port_byte);
}

int IspPortData(int b)
{
    if(b) 
        port_byte |= 0x20; 
    else 
        port_byte &= ~0x20;
    _outp(0x378, port_byte);
    //IspDelay();
    IspPortClock(1);
    b = _inp(0x378 + 1) & 0x40;
    IspPortClock(0);

    return b;
}

/*! 
 * \brief Set reset line.
 * 
 * \param b If 0 line is set low, otherwise high.
 */
void IspPortReset(int b)
{
    if(b) 
        port_byte |= 0x80; 
    else 
        port_byte &= ~0x80;
    _outp(0x378, port_byte);
}

/*!
 * \brief Enable the ISP port.
 */
void IspPortEnable(int b)
{
    if(b)
        port_byte &= ~0x4C;
    else 
        port_byte |= 0x4C;
    _outp(0x378, port_byte);
}

/*!
 * \brief Initialize the ISP port.
 */
int IspPortInit(void)
{
    HANDLE h;

    /*
     * Register with giveio for port access.
     */
    h = CreateFile("\\\\.\\giveio", GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(h == INVALID_HANDLE_VALUE) {
        fprintf(stderr, "Failed to open giveio.sys\n");
	    return -1;
    }
	CloseHandle(h);

    _outp(0x378 + 2, 0);
    port_byte = ~0x14;
    _outp(0x378, port_byte);

    return 0;
}

